@@ -15,18 +15,19 @@ import yaml
 import logging
 import os
 import click
+from datetime import datetime
 
 from jinja2 import Template
 from pykwalify.core import Core
 from betterboto import client as betterboto_client
 
-from servicecatalog_puppet import cli_command_helpers
-from servicecatalog_puppet import luigi_tasks_and_targets
-from servicecatalog_puppet import manifest_utils
-from servicecatalog_puppet import aws
+from . import cli_command_helpers
+from . import luigi_tasks_and_targets
+from . import manifest_utils
+from . import aws
 
-from servicecatalog_puppet import asset_helpers
-from servicecatalog_puppet import constants
+from . import asset_helpers
+from . import constants
 
 logger = logging.getLogger()
 logger.setLevel(logging.INFO)
@@ -121,20 +122,16 @@ def reset_provisioned_product_owner(f):
     cli_command_helpers.run_tasks(tasks_to_run, 10)
 
 
-def deploy(f, num_workers):
+def generate_tasks(f):
     puppet_account_id = cli_command_helpers.get_puppet_account_id()
-
     manifest = manifest_utils.load(f)
-
-    launch_tasks = {}
     tasks_to_run = []
 
     should_use_sns = cli_command_helpers.get_should_use_sns(os.environ.get("AWS_DEFAULT_REGION"))
 
-    all_launch_tasks = cli_command_helpers.deploy_launches(manifest, puppet_account_id)
-    launch_tasks.update(all_launch_tasks)
+    task_defs = manifest_utils.convert_manifest_into_task_defs_for_launches(manifest, puppet_account_id, should_use_sns)
 
-    for task in cli_command_helpers.wire_dependencies(launch_tasks):
+    for task in task_defs:
         task_status = task.get('status')
         del task['status']
         if task_status == constants.PROVISIONED:
@@ -154,18 +151,44 @@ def deploy(f, num_workers):
                     else:
                         raise Exception(f"Launch {task.get('launch_name')} has disallowed attribute: {attribute}")
 
+            del task['launch_parameters']
+            del task['manifest_parameters']
+            del task['account_parameters']
+            del task['should_use_sns']
+            del task['requested_priority']
+
             tasks_to_run.append(luigi_tasks_and_targets.TerminateProductTask(**task))
         else:
             raise Exception(f"Unsupported status of {task_status}")
 
-    spoke_local_portfolio_tasks_to_run = cli_command_helpers.deploy_spoke_local_portfolios(
-        manifest, launch_tasks, should_use_sns, puppet_account_id
+    tasks_to_run += manifest_utils.convert_manifest_into_task_defs_for_spoke_local_portfolios(
+        manifest, puppet_account_id, should_use_sns, tasks_to_run
     )
-    tasks_to_run += spoke_local_portfolio_tasks_to_run
+    return tasks_to_run
 
+
+def deploy(f, num_workers):
+    tasks_to_run = generate_tasks(f)
     cli_command_helpers.run_tasks(tasks_to_run, num_workers)
 
 
+def graph(f):
+    tasks_to_run = generate_tasks(f)
+    lines = []
+    nodes = []
+    for task in tasks_to_run:
+        lines += task.get_graph_lines()
+        nodes.append(task.graph_node())
+    print(json.dumps(lines))
+    click.echo("digraph G {\n")
+    click.echo("node [shape=record fontname=Arial];")
+    for node in nodes:
+        click.echo(f"{node};")
+    for line in lines:
+        click.echo(f"{line} [label=\"depends on\"];")
+    click.echo("}")
+
+
 def bootstrap_spoke_as(puppet_account_id, iam_role_arns):
     cross_accounts = []
     index = 0
@@ -664,3 +687,60 @@ def bootstrap_spokes_in_ou(ou_path_or_id, role_name, iam_role_arns):
                 )
 
         cli_command_helpers.run_tasks_for_bootstrap_spokes_in_ou(tasks)
+
+
+def handle_action_execution_detail(action_execution_detail):
+    action_type_id = action_execution_detail.get('input').get('actionTypeId')
+    if action_type_id.get('category') == "Build" and action_type_id.get('owner') == "AWS" and action_type_id.get(
+            'provider') == "CodeBuild":
+        external_execution_id = action_execution_detail.get('output').get('executionResult').get('externalExecutionId')
+
+        with betterboto_client.ClientContextManager(
+            "codebuild",
+            region_name=cli_command_helpers.get_home_region()
+        ) as codebuild:
+            builds = codebuild.batch_get_builds(ids=[external_execution_id]).get('builds')
+            build = builds[0]
+            log_details = build.get('logs')
+            with betterboto_client.ClientContextManager(
+                "logs",
+                region_name=cli_command_helpers.get_home_region()
+            ) as logs:
+                with open(
+                        f"log-{action_execution_detail.get('input').get('configuration').get('ProjectName')}.log", 'w'
+                ) as f:
+                    params = {
+                        'logGroupName':log_details.get('groupName'),
+                        'logStreamName':log_details.get('streamName'),
+                        'startFromHead':True,
+                    }
+                    has_more_logs = True
+                    while has_more_logs:
+                        get_log_events_response = logs.get_log_events(**params)
+                        if (len(get_log_events_response.get('events'))) > 0:
+                            params['nextToken'] = get_log_events_response.get('nextForwardToken')
+                        else:
+                            has_more_logs = False
+                            if params.get('nextToken'):
+                                del params['nextToken']
+                        for e in get_log_events_response.get('events'):
+                            d = datetime.utcfromtimestamp(e.get('timestamp')/1000).strftime('%Y-%m-%d %H:%M:%S')
+                            f.write(
+                                f"{d} : {e.get('message')}"
+                            )
+
+
+def export_puppet_pipeline_logs(execution_id):
+    with betterboto_client.ClientContextManager(
+            "codepipeline",
+            region_name=cli_command_helpers.get_home_region()
+    ) as codepipeline:
+        action_execution_details = codepipeline.list_action_executions(
+            pipelineName=constants.PIPELINE_NAME,
+            filter={
+                'pipelineExecutionId': execution_id
+            },
+        ).get('actionExecutionDetails')
+
+        for action_execution_detail in action_execution_details:
+            handle_action_execution_detail(action_execution_detail)
