@@ -1,7 +1,7 @@
 # Copyright 2019 Amazon.com, Inc. or its affiliates. All Rights Reserved.
 # SPDX-License-Identifier: Apache-2.0
 
-from servicecatalog_puppet import cli_commands
+from . import cli_commands
 
 
 def run(what="puppet", wait_for_completion=False):
